import { __awaiter, __generator, __rest } from "tslib";
export default function Model(context, model) {
    /**
     * Soft delete model(s) matching a condition.
     */
    model.softDelete = function (payload) {
        return this.dispatch('softDelete', payload);
    };
    /**
     * Soft delete a model instance.
     */
    model.prototype.$softDelete = function (hydrate) {
        return __awaiter(this, void 0, void 0, function () {
            var model, _a, key, flagName;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0: return [4 /*yield*/, this.$dispatch('softDelete', this.$self().getIdFromRecord(this))];
                    case 1:
                        model = _b.sent();
                        if (hydrate) {
                            this.$fill(model.$getAttributes());
                            return [2 /*return*/, this];
                        }
                        _a = context.createConfig(this.$self().softDeleteConfig), key = _a.key, flagName = _a.flagName;
                        this[key] = model[key];
                        this[flagName] = model[flagName];
                        return [2 /*return*/, model];
                }
            });
        });
    };
    /**
     * Restore a model instance.
     */
    model.prototype.$restore = function (hydrate) {
        return __awaiter(this, void 0, void 0, function () {
            var _a, key, flagName, model;
            var _b;
            return __generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        _a = context.createConfig(this.$self().softDeleteConfig), key = _a.key, flagName = _a.flagName;
                        return [4 /*yield*/, this.$dispatch('update', {
                                where: this.$self().getIdFromRecord(this),
                                data: (_b = {},
                                    _b[key] = null,
                                    _b[flagName] = false,
                                    _b)
                            })];
                    case 1:
                        model = _c.sent();
                        if (hydrate) {
                            this.$fill(model.$getAttributes());
                            return [2 /*return*/, this];
                        }
                        this[key] = model[key];
                        this[flagName] = model[flagName];
                        return [2 /*return*/, this];
                }
            });
        });
    };
    /**
     * Determine if the model instance has been soft deleted.
     */
    model.prototype.$trashed = function () {
        var flagName = context.createConfig(this.$self().softDeleteConfig).flagName;
        return this[flagName] === true;
    };
    /**
     * Soft-delete a model instance.
     * This method is deprecated and will warn users until retired.
     * @deprecated since v1.2.0
     */
    model.prototype.softDelete = function (hydrate) {
        /* istanbul ignore next */
        if (process.env.NODE_ENV !== 'production') {
            console.warn('The `softDelete` instance method has been deprecated. Please use `$softDelete`.');
        }
        return this.$softDelete(hydrate);
    };
    /**
     * Add supporting attributes to model.
     */
    var $fields = model.prototype.$fields;
    model.prototype.$fields = function () {
        var _a;
        var fields = $fields.call(this);
        var _b = context.createConfig(this.$self().softDeleteConfig), key = _b.key, flagName = _b.flagName;
        return Object.assign({}, fields, (_a = {},
            _a[key] = this.$self().attr(null),
            _a[flagName] = this.$self().attr(false),
            _a));
    };
    /**
     * Flags are visible by default during model serialization. They can be hidden
     * by setting `exposeFlagsExternally` to false.
     */
    var $toJson = model.prototype.$toJson;
    model.prototype.$toJson = function () {
        var toJson = $toJson.call(this);
        var config = context.createConfig(this.$self().softDeleteConfig);
        if (config.exposeFlagsExternally !== true) {
            /* istanbul ignore next */
            var _a = toJson, _b = config.key, k = _a[_b], _c = config.flagName, fn = _a[_c], json = __rest(_a, [typeof _b === "symbol" ? _b : _b + "", typeof _c === "symbol" ? _c : _c + ""]);
            return json;
        }
        return toJson;
    };
}
//# sourceMappingURL=Model.js.map